import re
from collections import Counter
import matplotlib.pyplot as plt

"""Read words from a file and return a list."""
def read_words(file_path='big.txt'):
    with open(file_path, 'r') as file:
        return re.findall(r'\w+', file.read().lower())

"""Calculate probabilities of words."""
def calculate_probabilities(word_counter):
    total_words = sum(word_counter.values())
    return {word: count / total_words for word, count in word_counter.items()}

"""Filter out words that are not in the given set."""
def known_words(words, word_set):
    return set(w for w in words if w in word_set)


"""Generate possible spelling corrections for a word."""
def generate_edits(word):
    letters = 'abcdefghijklmnopqrstuvwxyz'
    splits = [(word[:i], word[i:]) for i in range(len(word) + 1)]
    deletes = [L + R[1:] for L, R in splits if R]
    transposes = [L + R[1] + R[0] + R[2:] for L, R in splits if len(R) > 1]
    replaces = [L + c + R[1:] for L, R in splits if R for c in letters]
    inserts = [L + c + R for L, R in splits for c in letters]
    return set(deletes + transposes + replaces + inserts)

"""Find the most probable spelling correction for a word."""
def correction(word, word_counter):
    candidates = known_words([word], word_counter) or known_words(generate_edits(word), word_counter) or [word]
    return max(candidates, key=word_counter.get)

"""Count the corrections for a list of words."""
def count_corrections(words_to_analyze, word_counter):
    corrections_count = Counter()

    for word in words_to_analyze:
        corrected_word = correction(word, word_counter)
        if corrected_word != word:
            corrections_count[corrected_word] += 1

    return corrections_count

"""Analyze and plot the popularity of spelling correction candidates."""
def analyze_candidate_popularity(text, word_counter):
    words_to_analyze = re.findall(r'\w+', text.lower())

    candidates_popularity = Counter(correction(word, word_counter) for word in words_to_analyze)

    labels, values = zip(*candidates_popularity.items())
    plt.bar(labels, values)
    plt.xlabel('Correction Candidate')
    plt.ylabel('Occurrences')
    plt.title('Correction Candidate Popularity')
    plt.show()

"""Displays result output"""
def display_corrected_text(text, word_counter):
    words_to_analyze = re.findall(r'\w+', text.lower())

    corrected_text = ' '.join(correction(word, word_counter) for word in words_to_analyze)

    print("Original Text:", text)
    print("Corrected Text:", corrected_text)

word_counter = Counter(read_words())
sample_text = "Thiis is sampl text wth soome misspeled wrds"
analyze_candidate_popularity(sample_text, word_counter)
display_corrected_text(sample_text, word_counter)